# /*
# This file is part 1/2 of the RR secure discussions over irc
# implementation for ircII. The 2/2 part is the file: ur.irc
#
# Copyright (C) 1992,99 Bisqwit (http://iki.fi/bisqwit/)
#
# This plugin is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# GNU General Public License is available
# at http://www.gnu.org/copyleft/gpl.html
#
#
# This program is ANSI C and needs to be compiled.
# It's automatically invoked by ur.irc, the second
# part of this implementation.
# */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef __GNUC__
#include <unistd.h>
#endif

static const char *decode(const char *input)
{
    static char temp[2048];
    const char *c=input;
    char d,e;
    int i = 0;
    
    while((d = *c) && (e = *(c+1)))
    {
    	int a = ((d - 0x41) << 4) | (e - 0x41);
		temp[i++] = a;
        c += 2;
    }
    temp[i] = 0;
    return temp;
}


static const char urstr[] = "rRgh";   /* "rRgUhuGHb" if you want tinies */
static const int alku = 0;              /* x:lle (crypt) */
static const char loppu = 'G';          /* ei saa olla urstr:ss */
static const char *header = "r/";      /* header */
static const char escape = '';         /* for nicks */

static unsigned char XOR(unsigned char a)
{
	return a ^ 97;
}

static struct word
{
	char *s;
	struct word *next;
} *words = NULL;
static struct word *wlist = NULL;
static unsigned wcount;

#undef toupper
static int toupper(int a) { return (a<'a'||a>'z')?a:a+('A'-'a'); }
static int strequal(const char *s1, const char *s2, unsigned len)
{
	while(len)
	{
		if(toupper(*s1) != toupper(*s2))return 0;
		--len;
		++s1;
		++s2;
	}
	return len ? 0 : 1;
}

static int urlen;
static int en(const char *s)
{
	int x, sp=0, nks=1;
	printf(header);
	for(x=alku; *s; s++,x++)
	{
		unsigned char i;
		
		if(nks)
		{
			unsigned a, len=0, ind=0;
			for(a=0; a<wcount; a++)
			{
				unsigned b = strlen(wlist[a].s);
				if(b > len && strequal(s, wlist[a].s, b))
					len = b, ind = a;
			}
			if(len>2)
			{
				putchar(escape);
				if(sp)putchar(' ');
				for(; len; len--)putchar(*s++);
				putchar(escape);
				sp=0;
				putchar(' ');
			}
			nks=0;
		}
		
		i = XOR((unsigned char)(*s));
		
		if(!strchr("ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		           "abcdefghijklmnopqrstuvwxyz"
		           "0123456789_-`|[\\]", toupper(*s)))nks=1;
		
		if(!i && !s[1])
		{
			if(sp)putchar(loppu);
			putchar('.');
			break;
		}
		if(sp)
		{
			putchar(i ? ' ' : ',');
			sp=0;
		}
		while(i>0)
		{
			printf("%c", urstr[(i+x)%urlen]);
			i /= urlen;
		}
		sp=1;
	}
	printf("\n");
	return 0;
}

static void getwords(void)
{
	char Buf[512];
	unsigned w;
	struct word *p;
	for(w=0; scanf("%s", Buf) == 1; w++)
	{
		if(Buf[0]=='%')break; /* hack */
		p = (struct word *)malloc(sizeof(struct word));
		p->next = words;
		words = p;
		p->s = (char *)malloc(strlen(Buf)+1);
		strcpy(p->s, Buf);
	}
	wlist = (struct word *)malloc(w * (sizeof(struct word)));
	for(w=0, p=words; p; w++, p=p->next)wlist[w] = *p;
	wcount = w;
}

static int de(const char *s)
{
	int x;
	s += strlen(header);
	for(x=alku; *s; x++)
	{
		int k;
		unsigned char c, i;
		if(*s==escape)
		{
			if(*++s==' ')s++;
			while(*s)
			{
				if(*s==escape)break;
				putchar(*s++);
			}
			if(*s)s++;
			if(*s==' ')s++;
		}
		if(*s=='.')
		{
			putchar(XOR(0));
			break;
		}
		for(i=0, k=1; ; k*=urlen)
		{
			const char *u;
			u = strchr(urstr, *s);
			if(!u || !*s)break;
			c = (u - urstr);
			i += ((c+urlen-x%urlen)%urlen) * k;
			s++;
		}
		putchar((char)(XOR(i)));
		if(*s && *s!=escape)s++;
	}
	printf("\n");
	return 0;
}

// VERSION: 1.08

int main(int argc, const char **argv)
{
	const char *arg;
	int mode;
	if(argc != 2
	#ifdef __GNUC__
	 || isatty(fileno(stdin))
	#endif
	)
	{
	Usage:
		printf(
			"rrcrypter version 1.08\n"
			"Copyright (C) 1992,99 Bisqwit (http://iki.fi/bisqwit/)\n"
			"\n"
			"Usage: ur +$encode(clear text) < tokenwordlist\n"
			"Usage: ur '-rred text' < redirected-crap\n"
			"\n");
		return -1;
	}
	arg = argv[1];
	mode = *arg++;
	if(mode!='-' && mode!='+')goto Usage;
	urlen = strlen(urstr);
	
	if(mode=='+')
	{	
		getwords();
		return en(decode(arg));	
	}
	return de(arg);
}
